#!/bin/bash

s_echo()
{
	echo "[$(date)] $@" >> "$LOGFILE"
}

request()
{
	curl -A "$UA" -s --cookie-jar "$COOKIEFILE" -b "$COOKIEFILE" $@
}

login()
{
	s_echo "Downloading login form"

	if ! request "https://$DOMAIN/prisijungti" > /dev/null; then
		s_echo "Login failed ! (IE)"
		exit 1
	fi

	s_echo "Logging in as $USER"

	if ! curl -A "$UA" -s --cookie-jar "$COOKIEFILE" -b "$COOKIEFILE" -d "_method=POST&data[AppUser][email]=$USER&data[AppUser][password]=$PASS" -H "Content-Type: application/x-www-form-urlencoded" -X POST "https://$DOMAIN/prisijungti"; then
		s_echo "Login failed ! (IE)"
		exit 1
	fi
}

get_status_code()
{
	request -A "$UA" -I -X GET "$1" | head -n1 | awk -F" " '{print $2}'
}

logged_in()
{
	_int_status_code=$(get_status_code "https://$DOMAIN/profilis")

	if [ "$_int_status_code" -eq 200 ]; then
		return 0
	else
		return 1
	fi
}

_int_get_playlist_url()
{
	_int_chan=$1

	if ! logged_in; then
		login

		if ! logged_in; then
			s_echo "Login failed !"
			return 1
		fi
	fi

	s_echo "Get playlist for: $_int_chan"

	_int_chanurl="http://$DOMAIN/kanalas/$_int_chan"

	_int_data=$(request "$_int_chanurl")

	# redirect while logged in, possible N-18 check
	if [ -z "$_int_data" ]; then
		s_echo "Possible N-18 check detected, attempting to bypass."

		# get the destination URL
		_int_headers=$(request -I -X GET "$_int_chanurl" | grep -i "Location: " | sed -e "s/Location: //g")

		if [ -z "$_int_headers" ]; then
			s_echo "Failed !"
			return 1
		fi

		# convert it to a bypass URL
		_int_headers=$(sed -e "s#n18#n18/1#g" <<< "$_int_headers" | tr -d '\n' | tr -d "\r")

		if [ ! -z "$_int_headers" ]; then
			s_echo "Bypass URL: $_int_headers"

			_int_bypassurl="$_int_headers"
		else
			return 1
		fi

		# get channel page, follow redirect
		_int_data=$(request -L "$_int_bypassurl" --referer "$_int_bypassurl")

		#echo "$data"
	fi

	_int_url=$(grep -i "playlist" <<< "$_int_data" | awk -F\' '{for(i=1;i<NF;i++){if($i~/^http/){print $i}}}')

	echo "$_int_url"

	return 0
}

_int_get_chunklist_url() {
	_int_chan=$1

	_int_playlist=$(get_playlist "$_int_chan")

	if [ -z "$_int_playlist" ]; then
		return 1
	fi

	# first chunklist should have the highest bitrate
	_int_chunklist_fn=$(grep "chunklist" <<< "$_int_playlist" | head -n 1)

	if [ -z "$_int_chunklist_fn" ]; then
		s_echo "Got playlist without any chunklists for $_int_chan !"

		return 1
	fi

	_int_url=$(get_cached_file "$_int_chan" "playlist")
	_int_source_url=$(dirname "$_int_url")

	_int_chunklist_url="$_int_source_url/$_int_chunklist_fn"

	echo "$_int_chunklist_url"

	return 0
}

_int_get_template() {
	_int_chan=$1
	_int_type=$2
	_int_callback=$3
	_int_data=$(get_cached_url_data "$_int_chan" "$_int_type")

	if [ -z "$_int_data" ]; then
		s_echo "Updating cached $_int_type for $_int_chan"

		_int_url=$($_int_callback "$_int_chan")

		# order is important for $? !
		if [ $? -ne 0 ] || [ -z "$_int_url" ]; then
			s_echo "Failed to retrieve $_int_type for $_int_chan !"

			return 1
		fi

		if ! update_cached_file "$_int_chan" "$_int_type" "$_int_url"; then
			s_echo "Failed to update cached $_int_type for $_int_chan !"

			return 1
		fi

		_int_data=$(get_cached_url_data "$_int_chan" "$_int_type")

		if [ -z "$_int_data" ]; then
			s_echo "Failed to get $_int_type for $_int_chan"
		fi
	#else
	#	s_echo "Cache hit: $_int_type for $_int_chan"
	fi

	echo "$_int_data"
}

get_chunklist() {
	_int_chunklist=$(_int_get_template "$1" "chunklist" "_int_get_chunklist_url")

	# there has to be a better way to do this...
	_int_playlist_url=$(get_cached_file "$1" "playlist")
	_int_source_url=$(dirname "$_int_playlist_url")

	sed -e "s#media#$_int_source_url/media#g" <<< "$_int_chunklist"
}

get_playlist() {
	_int_get_template "$1" "playlist" "_int_get_playlist_url"
}
